@@ -1,24 +1,27 @@
 #!/usr/bin/python3
-import apt
 import gettext
-import gi
 import hashlib
 import locale
 import os
 import stat
-import subprocess
 import sys
 import tarfile
 import threading
 import time
+
+import gi
 gi.require_version("Gtk", "3.0")
 gi.require_version("XApp", "1.0")
 from gi.repository import Gtk, GdkPixbuf, Gio, GLib, XApp
 
+import apt_pkg
 import aptdaemon.client
-from aptdaemon.enums import *
-from aptdaemon.gtk3widgets import AptErrorDialog, AptConfirmDialog, AptProgressDialog, AptStatusIcon
 import aptdaemon.errors
+from aptdaemon.enums import *
+from aptdaemon.gtk3widgets import (AptConfirmDialog, AptErrorDialog,
+                                   AptProgressDialog, AptStatusIcon)
+
+from mintcommon.installer.cache import PkgCache
 
 import setproctitle
 setproctitle.setproctitle("mintbackup")
@@ -98,10 +101,13 @@ class MintBackup:
         self.excludes_model = Gtk.ListStore(str, GdkPixbuf.Pixbuf, str)
         self.excludes_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         treeview.set_model(self.excludes_model)
+        excluded_paths = []
         self.excludes_model.append([BACKUP_DIR[len(self.home_directory) + 1:], self.dir_icon, BACKUP_DIR])
+        excluded_paths.append(BACKUP_DIR)
         for item in self.settings.get_strv("excluded-paths"):
             item = os.path.expanduser(item)
-            if os.path.exists(item):
+            if os.path.exists(item) and item not in excluded_paths:
+                excluded_paths.append(item)
                 if os.path.isdir(item):
                     self.excludes_model.append([item[len(self.home_directory) + 1:], self.dir_icon, item])
                 else:
@@ -109,6 +115,7 @@ class MintBackup:
         self.builder.get_object("button_add_file").connect("clicked", self.add_item_to_treeview, treeview, self.file_icon, Gtk.FileChooserAction.OPEN, False)
         self.builder.get_object("button_add_folder").connect("clicked", self.add_item_to_treeview, treeview, self.dir_icon, Gtk.FileChooserAction.SELECT_FOLDER, False)
         self.builder.get_object("button_remove_exclude").connect("clicked", self.remove_item_from_treeview, treeview)
+        self.builder.get_object("treeview_excludes_selection").connect("changed", self.on_treeview_excludes_selection_changed)
 
         # set up inclusions page
         treeview = self.builder.get_object("treeview_includes")
@@ -179,20 +186,20 @@ class MintBackup:
         t.append_column(c2)
 
         # choose a package list
-        t = self.builder.get_object("treeview_package_list")
-        self.builder.get_object("button_select_list").connect("clicked", self.set_selection, t, True, True)
-        self.builder.get_object("button_deselect_list").connect("clicked", self.set_selection, t, False, True)
+        self.treeview_package_list = self.builder.get_object("treeview_package_list")
+        self.builder.get_object("button_select_list").connect("clicked", self.set_selection, self.treeview_package_list, True, True)
+        self.builder.get_object("button_deselect_list").connect("clicked", self.set_selection, self.treeview_package_list, False, True)
         self.builder.get_object("button_refresh").connect("clicked", self.restore_pkg_load_from_file)
         tog = Gtk.CellRendererToggle()
-        tog.connect("toggled", self.toggled_cb, t)
+        tog.connect("toggled", self.toggled_cb, self.treeview_package_list)
         c1 = Gtk.TreeViewColumn("", tog, active=0, activatable=2)
         c1.set_cell_data_func(tog, self.celldatamethod_checkbox)
-        t.append_column(c1)
+        self.treeview_package_list.append_column(c1)
         c2 = Gtk.TreeViewColumn("", Gtk.CellRendererText(), markup=1)
-        t.append_column(c2)
+        self.treeview_package_list.append_column(c2)
 
         file_filter = Gtk.FileFilter()
-        file_filter.add_pattern ("*.list");
+        file_filter.add_pattern ("*.list")
         filechooser = self.builder.get_object("filechooserbutton_package_source")
         filechooser.connect("file-set", self.restore_pkg_validate_file)
         filechooser.set_filter(file_filter)
@@ -201,8 +208,12 @@ class MintBackup:
         self.builder.get_object("filechooserbutton_backup_dest").set_current_folder(BACKUP_DIR)
         self.builder.get_object("filechooserbutton_package_source").set_current_folder(BACKUP_DIR)
 
+    def on_treeview_excludes_selection_changed(self, selection):
+        liststore, treeiter = selection.get_selected()
+        self.builder.get_object("button_remove_exclude").set_sensitive((treeiter and liststore.get_value(treeiter, 2) != BACKUP_DIR))
+
     def show_message(self, message, message_type=Gtk.MessageType.WARNING):
-        dialog = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT, message_type, Gtk.ButtonsType.OK, message)
+        dialog = Gtk.MessageDialog(self.main_window, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT, message_type, Gtk.ButtonsType.OK, message)
         dialog.set_title(_("Backup Tool"))
         dialog.set_position(Gtk.WindowPosition.CENTER)
         dialog.run()
@@ -210,7 +221,7 @@ class MintBackup:
 
     def add_item_to_treeview(self, widget, treeview, icon, mode, show_hidden=False):
         # Add a file or directory to treeview
-        dialog = Gtk.FileChooserDialog(_("Backup Tool"), None, mode, (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+        dialog = Gtk.FileChooserDialog(_("Backup Tool"), self.main_window, mode, (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
         dialog.set_current_folder(self.home_directory)
         dialog.set_select_multiple(True)
         dialog.set_show_hidden(show_hidden)
@@ -679,9 +690,18 @@ class MintBackup:
         model = Gtk.ListStore(bool, str, str)
         model.set_sort_column_id(1, Gtk.SortType.ASCENDING)
 
-        cache = apt.Cache()
-        settings = Gio.Settings("com.linuxmint.install")
-        for item in settings.get_strv("installed-apps"):
+        pkgcache = PkgCache()
+        installed_packages = pkgcache.get_manually_installed_packages()
+        if not installed_packages:
+            settings = Gio.Settings("com.linuxmint.install")
+            installed_packages = settings.get_strv("installed-apps")
+        else:
+            self.builder.get_object("label_caption_software_backup2").set_text(_("The list below shows the applications you installed."))
+
+        apt_pkg.init()
+        cache = apt_pkg.Cache()
+        package_records = apt_pkg.PackageRecords(cache)
+        for item in installed_packages:
             try:
                 if item.startswith(("apt:", "fp:")):
                     # Split package hash at first ':' since some packages have ':i386' suffixes
@@ -692,11 +712,10 @@ class MintBackup:
                     name = item
                 if prefix == "apt" and name in cache:
                     pkg = cache[name]
-                    if pkg.is_installed:
-                        desc = pkg.name + "\n<small>" + pkg.installed.summary.replace("&", "&amp;") + "</small>"
-                    elif pkg.candidate is not None:
-                        desc = pkg.name + "\n<small>" + pkg.candidate.summary.replace("&", "&amp;") + "</small>"
-                    model.append([True, pkg.name, desc])
+                    if pkg.current_ver:
+                        package_records.lookup(pkg.version_list[0].translated_description.file_list[0])
+                        desc = "%s\n<small>%s</small>" % (pkg.name, GLib.markup_escape_text(package_records.short_desc))
+                        model.append([True, pkg.name, desc])
             except Exception as e:
                 print(e)
         self.builder.get_object("treeview_packages").set_model(model)
@@ -736,42 +755,57 @@ class MintBackup:
                 for line in source:
                     line = line.rstrip("\r\n")
                     if line != "":
-                        if not line.endswith("\tinstall"):
+                        if not line.endswith("\tinstall") and not line.endswith(" install"):
                             self.show_message(_("The selected file is not a valid software selection."))
                             self.builder.get_object("button_forward").set_sensitive(False)
                             return
             self.builder.get_object("button_forward").set_sensitive(True)
         except Exception as detail:
             self.show_message(_("An error occurred while reading the file."))
+            print (detail)
 
+    @print_timing
     def restore_pkg_load_from_file(self, widget=None):
         # Load package list into treeview
         self.builder.get_object("button_forward").hide()
         self.builder.get_object("button_apply").show()
         self.builder.get_object("button_apply").set_sensitive(True)
         model = Gtk.ListStore(bool, str, bool, str)
-        self.builder.get_object("treeview_package_list").set_model(model)
+        self.treeview_package_list.set_model(model)
         try:
-            with open(self.package_source, "r") as source:
-                cache = apt.Cache()
-                for line in source:
-                    line = line.rstrip("\r\n")
-                    if line.startswith("#") or line == "":
+            with open(self.package_source, "r") as f:
+                source = f.readlines()
+            apt_pkg.init()
+            cache = apt_pkg.Cache()
+            package_records = apt_pkg.PackageRecords(cache)
+            depcache = apt_pkg.DepCache(cache)
+            for line in source:
+                try:
+                    if not line.strip() or line.startswith("#"):
+                        continue
+                    name = line.strip().replace(" install", "").replace("\tinstall", "")
+                    if not name:
                         continue
-                    name = line.split("\t")[0]
                     error = "%s\n<small>%s</small>" % (name, _("Could not locate the package."))
                     if name in cache:
                         pkg = cache[name]
-                        if not pkg.is_installed:
-                            if pkg.candidate is not None:
-                                status = "%s\n<small>%s</small>" % (name, pkg.candidate.summary.replace("&", "&amp;"))
+                        if not pkg.current_ver:
+                            candidate = depcache.get_candidate_ver(pkg)
+                            if candidate and candidate.downloadable:
+                                package_records.lookup(candidate.translated_description.file_list[0])
+                                summary = package_records.short_desc
+                                status = "%s\n<small>%s</small>" % (name, GLib.markup_escape_text(summary))
                                 model.append([True, status, True, pkg.name])
                             else:
                                 model.append([False, error, False, pkg.name])
                     else:
                         model.append([False, error, False, error])
+                except Exception as inner_detail:
+                    print("Error while reading '%s'." % line.strip())
+                    print(inner_detail)
         except Exception as detail:
             self.show_message(_("An error occurred while reading the file."))
+            print (detail)
         if len(model) == 0:
             self.builder.get_object("button_forward").hide()
             self.builder.get_object("button_back").hide()
